// BackupServlet.java - Generates backup and restore FTP scripts.
//
// Copyright (C) 1999,2000  Smart Software Consulting
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Smart Software Consulting
// 1688 Silverwood Court
// Danville, CA  94526-3079
// USA
//
// http://www.smartsc.com
//

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.smartsc.http.AuthenticatedHttpServlet;
import com.smartsc.util.AbsoluteFile;

public class BackupServlet
extends AuthenticatedHttpServlet
{
	public void init()
	{
		username = getInitParameter( "username");
		if( username == null)
		{
			username = "";
		}

		password = getInitParameter( "password");
		if( password == null)
		{
			password = "";
		}

		startDir = getInitParameter( "startDir");
		if( startDir == null)
		{
			startDir = "/";
		}
	}

	protected boolean authorized(
		HttpServletRequest req, String username, String password)
	{
		boolean authorized = false;
		if( this.username.equals( username)
		&&  this.password.equals( password))
		{
			authorized = true;
		}
		return authorized;
	}

	protected String realmName( HttpServletRequest req)
	{
		return "Backup Servlet";
	}

	protected void doAuthorizedGet(
		HttpServletRequest req, HttpServletResponse res)
	throws IOException
	{
		res.setContentType( "text/html");
		PrintWriter pw = res.getWriter();
		pw.println( "<HTML><HEAD><TITLE>Backup Servlet</TITLE></HEAD><BODY>");
		pw.println( "Create your FTP backup script in four easy steps...");
		pw.print  ( "<FORM ACTION=BackupServlet/text METHOD=POST ");
//		pw.print  ( "<FORM ACTION=SnoopServlet METHOD=POST ");
		pw.println( "ENCTYPE=application/x-www-form-urlencoded>");
		pw.println( "<OL><LI>Select the type of script you want.");
		pw.print  ( "<P><INPUT TYPE=RADIO NAME=type ");
		pw.println( "VALUE=backup CHECKED>Backup");
		pw.println( "<P><INPUT TYPE=RADIO NAME=type VALUE=restore>Restore");
		pw.println( "<P><HR><LI>Enter FTP commands to be issued at ");
		pw.println( "beginning of the backup/restore script. This can ");
		pw.println( "include the FTP username and password, if desired.");
		pw.println( "<P><TEXTAREA NAME=preface ROWS=7 COLS=35>root");
		pw.println( "tini");
		pw.println( "!mkdir backup");
		pw.println( "lcd backup</TEXTAREA>");
		pw.println( "<P><HR><LI>Enter the command used to make a ");
		pw.println( "<B>local</B> directory from within your FTP client. ");
		pw.println( "(Only needed for backup scripts.)");
		pw.println( "<P><INPUT TYPE=TEXT NAME=mkdir SIZE=25 VALUE=!mkdir>");
		pw.println( "<P><HR><LI>Click the <B>Create Script</B> button.");
		pw.println( "<P><INPUT TYPE=SUBMIT NAME=S VALUE=\"Create Script\">");
		pw.println( "</OL></FORM></BODY></HTML>");
	}

	protected void doAuthorizedPost(
		HttpServletRequest req, HttpServletResponse res)
	throws IOException
	{
		boolean isRestore = "restore".equals( req.getParameter( "type"));
		String preface = req.getParameter( PREFACE);
		String mkdir = req.getParameter( MKDIR);
		if( mkdir == null)
		{
			mkdir = MKDIR;
		}
		String action = "get";

		if( isRestore)
		{
			mkdir = MKDIR;
			action = "put";
		}

		res.setContentType( "text/plain");
		PrintWriter pw = res.getWriter();
		if( preface != null)
		{
			pw.println( preface);
		}

		pw.println( "binary");

		File f = AbsoluteFile.create( startDir);
		if( f.exists())
		{
			dumpTree( pw, f, mkdir, action);
		}

		pw.println( "close");
		pw.println( "quit");
	}

	protected void dumpTree(
		PrintWriter pw, File dir, String mkdir, String action)
	throws IOException
	{
		// In case initial call is for non-directory
		if( dir.isFile())
		{
			dumpCommand( pw, action, dir.getName());
			return;
		}

		String[] filenames = dir.list();
		for( int i = 0; i < filenames.length; ++i)
		{
			File f = new File( dir, filenames[i]);
			if( f.isFile())
			{
				dumpCommand( pw, action, filenames[i]);
			}
			else
			{
				dumpCommand( pw, mkdir, filenames[i]);
				dumpCommand( pw, "lcd", filenames[i]);
				dumpCommand( pw, "cd", filenames[i]);

				dumpTree( pw, f, mkdir, action);

				dumpCommand( pw, "lcd", UP_DIR);
				dumpCommand( pw, "cd", UP_DIR);
			}
		}
	}

	protected void dumpCommand( PrintWriter pw, String action, String target)
	{
		pw.print( action);
		pw.print( " ");
		pw.println( target);
	}


	private String username;
	private String password;
	private String startDir;

	public static final String PREFACE = "preface";
	public static final String MKDIR   = "mkdir";
	public static final String UP_DIR  = "..";
}
