// SnoopServlet.java - A request-snooping servlet.
//
// Copyright (C) 1999,2000  Smart Software Consulting
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// Smart Software Consulting
// 1688 Silverwood Court
// Danville, CA  94526-3079
// USA
// 
// http://www.smartsc.com
//

import java.io.*;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;


public class SnoopServlet
extends HttpServlet
{
	public void doPost( HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
	{
		doGet( req, res);
	}

	public void doGet( HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
	{
		PrintWriter	out;

		res.setContentType( "text/html");

		int i;
		Cookie[] cookies = req.getCookies();
		for( i = 0; i < cookies.length; ++i)
		{
			if( COOKIE_NAME.equals( cookies[i].getName()))
				break;
		}
		
		// If cookie not found
		boolean noCookie = (i == cookies.length);
		if( noCookie)
		{
			// Build cookie
			Cookie cookie = new Cookie( COOKIE_NAME, "Snooped!");
			cookie.setComment( "Test Cookie");
			cookie.setMaxAge( 60 * 60); // One hour
			cookie.setPath( req.getServletPath());
			cookie.setSecure( false);
			cookie.setVersion( 0);
			// Add cookie to response
			res.addCookie( cookie);
		}

		// Get writer
		out = res.getWriter();
			

		out.println( "<html>");
		out.println( "<head><title>Snoop Servlet</title></head>");
		out.println( "<body>");
		
		if( !noCookie)
		{
			out.println( "<h1>Welcome back!</h1>");
		}

		out.println( "<h1>Requested URL:</h1>");
		out.println( "<pre>");
		out.println( HttpUtils.getRequestURL( req).toString() );
		out.println( "</pre>");

		Enumeration enum = getServletConfig().getInitParameterNames();
		if( enum != null)
		{
			boolean first = true;
			while( enum.hasMoreElements())
			{
				if( first)
				{
					out.println( "<h1>Init Parameters</h1>");
					out.println( "<pre>");
					first = false;
				}
				String param = (String)enum.nextElement();
				out.println( " " + param + ": " + getInitParameter( param) );
			}
			out.println( "</pre>");
		}

		out.println( "<h1>Request information:</h1>");
		out.println( "<pre>");

		print( out, "Request method", req.getMethod());
		print( out, "Request URI", req.getRequestURI());
		print( out, "Request protocol", req.getProtocol());
		print( out, "Servlet path", req.getServletPath());
		print( out, "Path info", req.getPathInfo());
		print( out, "Path translated", req.getPathTranslated());
		print( out, "Query string", req.getQueryString());
		print( out, "Content length", req.getContentLength());
		print( out, "Content type", req.getContentType());
		print( out, "Server name", req.getServerName());
		print( out, "Server port", req.getServerPort());
		print( out, "Remote user", req.getRemoteUser());
		print( out, "Remote address", req.getRemoteAddr());
		print( out, "Remote host", req.getRemoteHost());
		print( out, "Authorization scheme", req.getAuthType());

		out.println( "</pre>");
	
		enum = req.getHeaderNames();
		if( enum.hasMoreElements())
		{
			out.println( "<h1>Request headers:</h1>");
			out.println( "<pre>");
			while( enum.hasMoreElements())
			{
				String name = (String)enum.nextElement();
				out.println( " " + name + ": " + req.getHeader(name) );
			}
			out.println( "</pre>");
		}

		enum = req.getParameterNames();
		if( enum.hasMoreElements())
		{
			out.println( "<h1>Servlet parameters (Single Value style):</h1>");
			out.println( "<pre>");
			while( enum.hasMoreElements())
			{
				String name = (String)enum.nextElement();
				out.println( " " + name + " = " + req.getParameter(name) );
			}
			out.println( "</pre>");
		}

		enum = req.getParameterNames();
		if( enum.hasMoreElements())
		{
			out.println( "<h1>Servlet parameters (Multiple Value style):</h1>");
			out.println( "<pre>");
			while( enum.hasMoreElements())
			{
				String   name = (String)enum.nextElement();
				String[] vals = req.getParameterValues( name);
				if (vals != null)
				{
					out.print( "<b> " + name + " = </b>"); 
					out.println( vals[0]);
					for( i = 1; i<vals.length; i++)
						out.println( "           " + vals[i]);
				}
				out.println( "<p>");
			}
			out.println("</pre>");
		}
	
		out.println("</body></html>");
	}

	private void print( PrintWriter out, String name, String value)
	{
		out.print( " " + name + ": ");
		out.println( value == null ? "&lt;none&gt;" : value);
	}

	private void print( PrintWriter out, String name, int value)
	{
		out.print( " " + name + ": ");
		if (value == -1)
		{
			out.println( "&lt;none&gt;");
		}
		else
		{
			out.println( value);
		}
	}
	public static final String COOKIE_NAME = "SnoopServletCookie";
}
