// iButtonServlet.java - A demo iButton servlet for TINI.
//
// Copyright (C) 1999,2000  Smart Software Consulting
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Smart Software Consulting
// 1688 Silverwood Court
// Danville, CA  94526-3079
// USA
//
// http://www.smartsc.com
//

import java.io.*;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibutton.iButtonAccessProvider;
import com.ibutton.iButtonException;
import com.ibutton.adapter.DSPortAdapter;
import com.ibutton.container.iButtonContainer;
import com.ibutton.utils.Address;

import com.smartsc.ibutton.PortAdapterEnumerator;
import com.smartsc.ibutton.PortAdapterEnumerator.Callback;

public class iButtonServlet
extends HttpServlet
{
	public void doPost( HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
	{
		doGet( req, res);
	}

	public final void doGet( HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
	{
		String adapter = req.getParameter( ADAPTER);
		String port = req.getParameter( PORT);

		// Do HTTP header stuff
		res.setContentType( "text/html");

		// Get Writer
		PrintWriter pw = res.getWriter();

		// Start HTML
		pw.println( "<html>");
		pw.println( "<head><title>iButton Servlet</title></head>");
		pw.println( "<body>");

		if( adapter != null && port != null)
		{
			DSPortAdapter pa = null;
			try
			{
				pa = getAdapter( adapter, port);

				try { registeriButtonContainerClasses( pa); }
				catch( iButtonException ibe) { log( ibe.toString()); }

				sendPortAdapterDetail( pw, req, pa);
			}
			catch( Exception e)
			{
				sendAlert( pw, e);
				pw.println( "<P>Error: " + e.getMessage());
			}
		}
		else
		{
			sendPortAdapterList( pw, req);
		}

		// Finish HTML
		pw.println("</body></html>");
	}

	public static final void sendPortAdapterList(
		final PrintWriter pw, final HttpServletRequest req)
	{
		pw.println( "<H1>iButton Port and Port Adapter Information</H1>");

		pw.println( "This servlet is a 1-wire explorer.  It uses the ");
		pw.println( "iButton classes to determine all of the 1-wire port ");
		pw.println( "adapters connected to this system.");
		pw.println( "<P>");
		pw.println( "The list below shows the 1-wire port adapters ");
		pw.println( "connected to this system and the ports to which ");
		pw.println( "they are connected. Click on [Details] to find ");
		pw.println( "out more about a specific 1-wire network.");
		pw.println( "<UL>");

		int portAdapterCount = PortAdapterEnumerator.enumerate(
			new PortAdapterEnumerator.Callback()
			{
				public void forEachPortAdapter( DSPortAdapter pa, String p)
				{
						pw.print  ( "<LI>A <b>");
						pw.print  ( pa.getAdapterName());
						pw.println( "</b> port adapter ");
						pw.println( "is connected to port <b>");
						pw.print  ( p);
						pw.print  ( "</b>. [<A HREF=");
						pw.print  ( req.getServletPath());
						pw.print  ( "?");
						pw.print  (
							buildQueryString( pa.getAdapterName(), p, null) );
						pw.print  ( ">Details</A>]");
				}
			});

		pw.println( "</UL>");

		pw.print( "Detected ");
		pw.print( portAdapterCount);
		pw.print( " port adapter");
		if( portAdapterCount != 1) pw.print( "s");
		pw.println( ".");
	}

	public static final void sendPortAdapterDetail(
		PrintWriter pw, HttpServletRequest req, DSPortAdapter pa)
	throws iButtonException
	{
		pw.println( "<H1> iButton 1-Wire Information</H1>");

		pw.println( "This page displays information about a particular ");
		pw.println( "1-wire network connected to this system.  The ");
		pw.println( "information is determined by using the iButton classes.");
		pw.println( "On any given system, a 1-wire network can be uniquely ");
		pw.println( "identified by specifying a port and a port adapter.");

		pw.println( "<H2>Port Information</H2>");

		String port = pa.getPortName();
		print( pw, "Name", port);
		print( pw, "Description", pa.getPortTypeDescription());

		pw.println( "<H2>Port Adapter Information</H2>");

		pw.println( "<H3>Logical Information</H3>");

		String adapter = pa.getAdapterName();
		print( pw, "Name", adapter);
		print( pw, "Class version", pa.getClassVersion());

		pw.println( "<H3>Physical Information</H3>");

		print( pw, "ID", pa.getAdapterAddress());
		print( pw, "Version", pa.getAdapterVersion());

		pw.println( "<H3>Power Information</H3>");

		print( pw, "Can deliver power", pa.canDeliverPower());
		print( pw, "Can deliver smart power", pa.canDeliverSmartPower());
		print( pw, "Can program", pa.canProgram());

		pw.println( "<H3>Speed Information</H3>");

		print( pw, "Can overdrive", pa.canOverdrive());
		print( pw, "Can hyperdrive", pa.canHyperdrive());

		pw.println( "<H2>Currently Detected iButtons</H2>");
		pw.println( "If a device specific servlet is available for a ");
		pw.println( "detected iButton, the iButton's ROM ID ");
		pw.println( "will be a link to the device specific servlet.");
		pw.println( "<DL>");

		int count = 0;

        // Clear any previous search restrictions
        pa.setSearchAlliButtons();
        pa.targetAllFamilies();
		Enumeration ibEnum = pa.getAlliButtons();
		while( ibEnum.hasMoreElements())
		{
			iButtonContainer ib =
				//com.smartsc.ibutton.iButtonContainer.createiButton(
					(iButtonContainer)ibEnum.nextElement()
					//, pa)
					;

			pw.print( "<DT>");

			pw.print(
				getRomIdHtml(
					adapter, port, ib.getAddress() ) );

			pw.print( " &#150; ");
			pw.print( htmlEscape( ib.getiButtonPartName()));
			pw.print( " (");
			pw.print( htmlEscape( ib.getAlternateNames()));
			pw.println( ")");

			pw.print( "<DD>");
			pw.println( htmlEscape( ib.getDescription()));
			pw.println( "<P>");
			++count;
		}
		pw.println( "</DL>");

		pw.print( "Detected ");
		pw.print( count);
		pw.print( " iButton");
		if( count != 1) pw.print( "s");
		pw.print( ".");
	}

	public static final String getRomIdHtml(
		String adapter, String port, byte[] id)
	{
		StringBuffer sb = new StringBuffer();

		boolean familyCodeSupported = isFamilyCodeSupported( id);
		if( familyCodeSupported)
		{
			sb.append( "<A HREF=");
			sb.append( getFamilyCodeServletName( id));
			sb.append( "?");
			sb.append( buildQueryString( adapter, port, id));
			sb.append( ">");
		}
		else
		{
			sb.append( "<B>");
		}
		sb.append( Address.toString( id));
		if( familyCodeSupported)
		{
			sb.append( "</A>");
		}
		else
		{
			sb.append( "</B>");
		}

		return sb.toString();
	}

	protected static final DSPortAdapter getAdapter( String adapterName)
	throws iButtonException
	{
		Enumeration paEnum = iButtonAccessProvider.enumerateAllAdapters();

		while( paEnum.hasMoreElements())
		{
			DSPortAdapter pa = (DSPortAdapter)paEnum.nextElement();
			String paName = pa.getAdapterName();
			if( paName != null && paName.equals( adapterName))
				return pa;
		}

		throw new iButtonException( "Adapter " +
			adapterName + " is not supported on this platform.");
	}

	protected static final DSPortAdapter getAdapter(
		String adapterName, String port)
	throws iButtonException
	{
		DSPortAdapter pa = getAdapter( adapterName);
		if( pa.selectPort( port))
		{
			if( pa.adapterDetected())
			{
				// Caller must free port
				return pa;
			}
		}

		throw new iButtonException( "Adapter " +
			adapterName + " is not detected on port " + port + ".");
	}

	protected void registeriButtonContainerClasses( DSPortAdapter pa)
	throws iButtonException
	{
		pa.registeriButtonContainerClass(
			0x20, com.smartsc.ibutton.iButtonContainer20.class);
		pa.registeriButtonContainerClass(
			0xa0, com.smartsc.ibutton.iButtonContainer20.class);
	}

	protected static final void print(
		PrintWriter pw, String name, String value)
	{
		pw.print( htmlEscape( name));
		pw.print( ": ");
		pw.print( htmlEscape( value));
		pw.print( "<BR>");
	}

	protected static final void print(
		PrintWriter pw, String name, boolean value)
	{
		print( pw, name, String.valueOf( value));
	}

	protected static final void sendAlert( PrintWriter pw, Throwable t)
	{
		sendAlert( pw, t.getMessage());
	}

	protected static final void sendAlert( PrintWriter pw, String s)
	{
		pw.println( "<SCRIPT>");
		pw.print  ( "alert('");
		pw.print  ( jsEscape( s));
		pw.println( "');");
		pw.println( "</SCRIPT>");
	}

	public static final String jsEscape( String s)
	{
		int len = s.length();
		StringBuffer sb = new StringBuffer( len);
		for( int i = 0; i < len; ++i)
		{
			char c = s.charAt( i);
			switch( c)
			{
				case '\b': sb.append( "\\b"); break;
				case '\f': sb.append( "\\f"); break;
				case '\n': sb.append( "\\n"); break;
				case '\r': sb.append( "\\r"); break;
				case '\t': sb.append( "\\t"); break;
				case '\'': sb.append( "\\'"); break;
				case '\"': sb.append( "\\\""); break;
				default  : sb.append( c);
			}
		}

		return sb.toString();
	}

	public static final String htmlEscape( String s)
	{
		int len = s.length();
		StringBuffer sb = new StringBuffer( len);
		StringTokenizer st = new StringTokenizer( s, "<>", true);
		while( st.hasMoreTokens())
		{
			String token = st.nextToken();
			if( token.equals( "<"))
				sb.append( "&lt;");
			else if( token.equals( ">"))
				sb.append( "&gt;");
			else
				sb.append( token);
		}
		return sb.toString();
	}

	protected static String buildQueryString(
		String adapter, String port, byte[] iButton)
	{
		StringBuffer sb = new StringBuffer( '?');
		buildQueryString( sb, ADAPTER, adapter);
		buildQueryString( sb, PORT,    port);
		if( iButton != null)
		{
			buildQueryString( sb, IBUTTON, Address.toString( iButton));
		}
		return sb.toString();
	}

	protected static final String buildQueryString( String name, String value)
	{
		StringBuffer sb = new StringBuffer();
		return buildQueryString( sb, name, value);
	}

	protected static final String buildQueryString(
		StringBuffer sb, String name, String value)
	{
		if( name != null && value != null)
		{
			if( sb.length() > 2) sb.append( '&');
			sb.append( name);
			sb.append( '=');
			sb.append( value);
		}
		return sb.toString();
	}

	public static final String getFamilyCodeString( byte[] id)
	{
		StringBuffer sb = new StringBuffer( 2);
		sb.append( (char)('0' + ((id[0]&0xf0) >> 4) ) );
		sb.append( (char)('0' +  (id[0]&0x0f)       ) );
		return sb.toString();
	}

	public static final String getFamilyCodeServletName( byte[] id)
	{
		StringBuffer sb = new StringBuffer( "FamilyCode");
		sb.append( getFamilyCodeString( id));
		sb.append( "Servlet");
		return sb.toString();
	}

	protected static final boolean isFamilyCodeSupported( byte[] id)
	{
		try
		{
			Class.forName( getFamilyCodeServletName( id));
			return true;
		}
		catch( ClassNotFoundException cnfe)
		{
			return false;
		}
	}

	public static final String ADAPTER = "adapter";
	public static final String PORT    = "port";
	public static final String IBUTTON = "iButton";
}
